package org.implab.gradle.containers.tasks;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutionException;

import org.gradle.api.GradleException;
import org.gradle.api.file.RegularFile;
import org.gradle.api.file.RegularFileProperty;
import org.gradle.api.provider.Property;
import org.gradle.api.provider.Provider;
import org.gradle.api.tasks.Internal;
import org.gradle.api.tasks.TaskAction;
import org.implab.gradle.containers.cli.Utils;

public abstract class StopContainer extends DockerCliTask {

    @Internal
    public abstract RegularFileProperty getContainerIdFile();

    @Internal
    public abstract Property<String> getContainerName();

    @Internal
    public abstract Property<String> getStopSignal();

    @Internal
    public abstract Property<Integer> getStopTimeout();

    Provider<String> readCid(RegularFile regularFile) {
        var file = regularFile.getAsFile();

        return provider(() -> {
            try {
                return file.isFile() ? Utils.readAll(file) : null;
            } catch (IOException e) {
                throw new GradleException("Failed to read container id from file " + file.toString(), e);
            }
        });
    }

    @TaskAction
    public void run() throws InterruptedException, IOException, ExecutionException {
        var cid = getContainerIdFile().flatMap(this::readCid).orElse(getContainerName());

        if (!cid.isPresent()) {
            getLogger().info("The container name or id hasn't been specified");
            return;
        }

        var options = new ArrayList<String>();

        if (getStopSignal().isPresent())
            options.addAll(List.of("--signal", getStopSignal().get()));

        if (getStopTimeout().isPresent())
            options.addAll(List.of("--time", getStopTimeout().map(Object::toString).get()));

        docker().stopContainer(cid.get(), options);

        if (getContainerIdFile().isPresent())
            getContainerIdFile().getAsFile().get().delete();
    }

}
